"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = _default;

require("@babel/polyfill");

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

var _url = _interopRequireDefault(require("url"));

var _markdownToAst = require("@textlint/markdown-to-ast");

var _astTraverse = require("@textlint/ast-traverse");

var _githubSlugger = _interopRequireDefault(require("github-slugger"));

var _util = _interopRequireDefault(require("util"));

var _textlintRuleHelper = require("textlint-rule-helper");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var fileRead = _util["default"].promisify(_fs["default"].readFile); //https://stackoverflow.com/a/31991870


var externalLinkRegex = new RegExp(/^[a-z][a-z0-9+.-]*:/, 'i');

function _default(context, options) {
  return (0, _textlintRuleHelper.wrapReportHandler)(context, {
    ignoreNodeTypes: [_markdownToAst.Syntax.code]
  }, function () {
    return handler(context, options);
  });
}

function handler(context, options) {
  return _defineProperty({}, context.Syntax.Link, function (linkNode) {
    return validateLinkNode(linkNode, context, options);
  });
}

function validateLinkNode(_x, _x2, _x3) {
  return _validateLinkNode.apply(this, arguments);
}

function _validateLinkNode() {
  _validateLinkNode = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee(linkNode, context, options) {
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            if (!(!linkNode.url || externalLinkRegex.test(linkNode.url))) {
              _context.next = 4;
              break;
            }

            return _context.abrupt("return");

          case 4:
            if (!(linkNode.url[0] === '#')) {
              _context.next = 8;
              break;
            }

            return _context.abrupt("return", validateAnchorLink(context.getFilePath(), linkNode.url.slice(1), linkNode, context));

          case 8:
            return _context.abrupt("return", validateRelativeLink(linkNode, context, options));

          case 9:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));
  return _validateLinkNode.apply(this, arguments);
}

function validateRelativeLink(_x4, _x5, _x6) {
  return _validateRelativeLink.apply(this, arguments);
}

function _validateRelativeLink() {
  _validateRelativeLink = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee2(linkNode, context, options) {
    var linkURL, routedLinkURL;
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            linkURL = getLinkURL(linkNode.url, context, options);
            _context2.next = 3;
            return fileExists(_url["default"].fileURLToPath(linkURL));

          case 3:
            if (_context2.sent) {
              _context2.next = 22;
              break;
            }

            if (!options["route-map"]) {
              _context2.next = 20;
              break;
            }

            _context2.next = 7;
            return getRoutedLink(linkNode, context, options);

          case 7:
            routedLinkURL = _context2.sent;

            if (routedLinkURL) {
              _context2.next = 13;
              break;
            }

            reportError(linkNode, context, "".concat(_path["default"].basename(linkURL.pathname), " has no mapped routing"));
            return _context2.abrupt("return");

          case 13:
            _context2.next = 15;
            return fileExists(_url["default"].fileURLToPath(routedLinkURL));

          case 15:
            if (_context2.sent) {
              _context2.next = 18;
              break;
            }

            reportError(linkNode, context, "The routed destination for ".concat(_path["default"].basename(routedLinkURL.pathname), " does not exist"));
            return _context2.abrupt("return");

          case 18:
            _context2.next = 22;
            break;

          case 20:
            reportError(linkNode, context, "".concat(_path["default"].basename(linkURL.pathname), " does not exist"));
            return _context2.abrupt("return");

          case 22:
            // use the routedLinkURL to check for the anchor
            if (routedLinkURL) {
              linkURL = routedLinkURL;
            }

            if (!(linkURL && linkURL.hash && _path["default"].extname(linkURL.pathname) === ".md")) {
              _context2.next = 25;
              break;
            }

            return _context2.abrupt("return", validateAnchorLink(_url["default"].fileURLToPath(linkURL), linkURL.hash.slice(1), linkNode, context));

          case 25:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));
  return _validateRelativeLink.apply(this, arguments);
}

function getRoutedLink(_x7, _x8, _x9) {
  return _getRoutedLink.apply(this, arguments);
}

function _getRoutedLink() {
  _getRoutedLink = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee3(linkNode, context, options) {
    var linkRouteMaps, nodeUrl, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, mapping, sourceRegex, mappedDestination, routedUrl;

    return regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            linkRouteMaps = options["route-map"];
            nodeUrl = linkNode.url;
            _iteratorNormalCompletion = true;
            _didIteratorError = false;
            _iteratorError = undefined;
            _context3.prev = 5;
            _iterator = linkRouteMaps[Symbol.iterator]();

          case 7:
            if (_iteratorNormalCompletion = (_step = _iterator.next()).done) {
              _context3.next = 17;
              break;
            }

            mapping = _step.value;
            sourceRegex = new RegExp(mapping["source"], "g");
            mappedDestination = mapping["destination"];

            if (!sourceRegex.test(nodeUrl)) {
              _context3.next = 14;
              break;
            }

            routedUrl = nodeUrl.replace(sourceRegex, mappedDestination);
            return _context3.abrupt("return", getLinkURL(routedUrl, context, options));

          case 14:
            _iteratorNormalCompletion = true;
            _context3.next = 7;
            break;

          case 17:
            _context3.next = 23;
            break;

          case 19:
            _context3.prev = 19;
            _context3.t0 = _context3["catch"](5);
            _didIteratorError = true;
            _iteratorError = _context3.t0;

          case 23:
            _context3.prev = 23;
            _context3.prev = 24;

            if (!_iteratorNormalCompletion && _iterator["return"] != null) {
              _iterator["return"]();
            }

          case 26:
            _context3.prev = 26;

            if (!_didIteratorError) {
              _context3.next = 29;
              break;
            }

            throw _iteratorError;

          case 29:
            return _context3.finish(26);

          case 30:
            return _context3.finish(23);

          case 31:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3, null, [[5, 19, 23, 31], [24,, 26, 30]]);
  }));
  return _getRoutedLink.apply(this, arguments);
}

function getLinkURL(nodeURL, context, options) {
  var linkAbsolutePath = _path["default"].resolve(_path["default"].dirname(context.getFilePath()), nodeURL);

  var linkURL = new URL("file://" + linkAbsolutePath);

  var linkedFileExtension = _path["default"].extname(linkURL.pathname);

  if (linkedFileExtension !== ".md" && options["resolve-as-markdown"] && options["resolve-as-markdown"].includes(linkedFileExtension)) {
    linkURL.pathname = linkURL.pathname.replace(linkedFileExtension, ".md");
  }

  return linkURL;
}

function fileExists(_x10) {
  return _fileExists.apply(this, arguments);
}

function _fileExists() {
  _fileExists = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee4(url) {
    var access;
    return regeneratorRuntime.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            access = _util["default"].promisify(_fs["default"].access);
            _context4.prev = 1;
            _context4.next = 4;
            return access(url);

          case 4:
            return _context4.abrupt("return", true);

          case 7:
            _context4.prev = 7;
            _context4.t0 = _context4["catch"](1);
            return _context4.abrupt("return", false);

          case 10:
          case "end":
            return _context4.stop();
        }
      }
    }, _callee4, null, [[1, 7]]);
  }));
  return _fileExists.apply(this, arguments);
}

function validateAnchorLink(_x11, _x12, _x13, _x14) {
  return _validateAnchorLink.apply(this, arguments);
}

function _validateAnchorLink() {
  _validateAnchorLink = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee5(filePath, anchor, linkNode, context) {
    var fileContent, ast, slugger, found;
    return regeneratorRuntime.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            _context5.next = 2;
            return fileRead(filePath, 'utf8');

          case 2:
            fileContent = _context5.sent;
            ast = (0, _markdownToAst.parse)(fileContent);
            slugger = new _githubSlugger["default"]();
            found = false;
            (0, _astTraverse.traverse)(ast, {
              enter: function enter(node) {
                if (node.type === _markdownToAst.Syntax.Header) {
                  var headerStr = node.raw.substring(node.depth).trim();
                  var id = slugger.slug(headerStr);

                  if (id === anchor) {
                    found = true;
                    return _astTraverse.VisitorOption.Break;
                  }
                }
              }
            });

            if (!found) {
              reportError(linkNode, context, "Anchor #".concat(anchor, " does not exist in ").concat(_path["default"].basename(filePath)));
            }

          case 8:
          case "end":
            return _context5.stop();
        }
      }
    }, _callee5);
  }));
  return _validateAnchorLink.apply(this, arguments);
}

function reportError(linkNode, context, errorMessage) {
  context.report(linkNode, new context.RuleError(errorMessage, {
    index: linkNode.raw.indexOf(linkNode.url) || 0
  }));
}
//# sourceMappingURL=no-dead-relative-link.js.map